#ifndef MOO_DOCUMENT_H
#define MOO_DOCUMENT_H

#include <moo/pimpl.h>
#include <QObject>
#include <QUrl>

namespace moo {

class Document;
class DocumentWindow;

class DocumentView {
public:
    virtual Document *document() = 0;
    virtual ~DocumentView() {}
};

class DocumentPrivate;
class Document : public QObject {
    Q_OBJECT
    M_DECLARE_IMPL(Document)
    friend class DocumentWindow;

public:
    enum DocumentState {
        Idle,
        Loading,
        Saving
    };

private:
    Q_PROPERTY(QString documentType READ documentType)
    Q_PROPERTY(QUrl url READ url)
    Q_PROPERTY(QString displayName READ displayName)
    Q_PROPERTY(QString displayBasename READ displayBasename)
    Q_PROPERTY(bool modified READ isModified WRITE setModified)
    Q_PROPERTY(bool empty READ isEmpty)
    Q_PROPERTY(bool untitled READ isUntitled)
    Q_PROPERTY(DocumentState documentState READ documentState)

private:
    void setUrl(const QUrl &url);
    void setDocumentState(DocumentState state);

protected:
    Document(QObject *parent);
    ~Document();

    virtual bool loadFile(const QUrl &url, QString &error) = 0;
    virtual bool saveFile(const QUrl &url, const QString &type, QString &error) = 0;

    bool load(const QUrl &url, QString &error);
    bool save(const QUrl &url, const QString &type, QString &error);

public:
    DocumentState documentState() const;
    bool isUntitled() const;
    QUrl url() const;
    QString displayName() const;
    QString displayBasename() const;
    virtual bool isModified() const = 0;
    virtual bool isEmpty() const;
    virtual void setModified(bool modified) = 0;
    virtual QString documentType() const = 0;
    virtual bool needSave() const;

Q_SIGNALS:
    void docStateChanged();
    void urlChanged(const QUrl &newUrl);
    void modificationChanged(bool);
};

}

#endif // MOO_DOCUMENT_H
